/*
 *
 * AttributeMgr
 *
 * Central point for the attribute skill system in the mod
 *
 *
*/

package jesse.keeblarcraft.AttributeMgr;

import java.util.HashMap;

import jesse.keeblarcraft.Keeblarcraft;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AbstractNode;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AttributeFlight;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AttributeMetalJacket;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import jesse.keeblarcraft.Utils.ChatUtil;
import net.minecraft.server.network.ServerPlayerEntity;

public class AttributeMgr {
    ConfigManager config;

    // Global list of attributes
    public static final HashMap<String, AbstractNode> attributes = new HashMap<String, AbstractNode>();

    // This is a list of all logged in player tree's. These are kept in memory until either the mod is torn down or
    // players log out for optimization reasons
    public static final HashMap<String, AttributeTree> activeTrees = new HashMap<String, AttributeTree>();

    @SuppressWarnings("deprecation")
    public static void RegisterAttributeClass(Class<? extends AbstractNode> classObj) {
        AbstractNode test;
        ChatUtil.LoggerColored("Registring attribute called", ChatUtil.CONSOLE_COLOR.CYAN, Keeblarcraft.LOGGER);
        try {
            test = classObj.newInstance();
            if (attributes.containsKey(test.GetNodeTitle())) {
                Keeblarcraft.LOGGER.warn("Could not register attribute with duplicate name '" + test.GetNodeTitle() + "'");
            } else {
                ChatUtil.LoggerColored("REGISTERING ATTRIBUTE " + test.GetNodeTitle(), ChatUtil.CONSOLE_COLOR.YELLOW, Keeblarcraft.LOGGER);
                attributes.put(test.GetNodeTitle(), test);
            }
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    public static String ApplyAttribute(String uuid, String attributeName) {
        String msg = "";
        if (attributes.containsKey(attributeName)) {
            AttributeTree playerTree = activeTrees.get(uuid);
            AbstractNode node = attributes.get(attributeName);

            if (playerTree != null) {
                ///////////
                // debug testing
                String isNull = (node == null ? "NULL" : "NOT NULL");
                System.out.println("Node is " + isNull);
                if (isNull.equals("NOT NULL")) { System.out.println("Node name: " + node.GetNodeTitle()); }
                // end debug testing
                ///////////

                playerTree.AddNewNode(node, 1, null, null);
                msg = "Applied attribute '" + attributeName + "' successfully";
            } else {
                msg = "Player tree not found!";
            }
        } else {
            msg = "Could not apply attribute, attribute name does not exist!";
        }
        return msg;
    }

    public static void RegisterAttributes() {
        // Manually register all attribute node classes here
        /// TODO: Find a better way to do this more dynamically in the future

        RegisterAttributeClass(AttributeFlight.class);
        RegisterAttributeClass(AttributeMetalJacket.class);
    }
}
